from typing import Optional

from PyEmailerAJM.continuous_monitor import ContinuousMonitor
from PyEmailerAJM.backend import EmailMsgImportanceLevel

# This is installed as part of pywin32
# noinspection PyUnresolvedReferences
from pythoncom import com_error

NO_COLORIZER = False


class ContinuousMonitorAlertSend(ContinuousMonitor):
    ADMIN_EMAIL_LOGGER = []
    ADMIN_EMAIL = []
    DEFAULT_SUBJECT = "Email Alert"
    DEFAULT_MSG_BODY = ("Dear {admin_email_names},\n\n"
                        "There is an Email in the inbox that has an alert ({msg_tuple}). \n\n"
                        "Thanks,\n"
                        "{email_sender}")
    ATTRS_TO_CHECK = ['ADMIN_EMAIL', 'ADMIN_EMAIL_LOGGER']
    ALERT_EMAIL_IMPORTANCE = EmailMsgImportanceLevel.HIGH
    DEFAULT_EMAIL_IMPORTANCE = EmailMsgImportanceLevel.NORMAL

    def __init__(self, display_window: bool, send_emails: bool, **kwargs):

        super().__init__(display_window, send_emails, **kwargs)
        if not self.dev_mode:
            if type(self) is ContinuousMonitorAlertSend:
                self.__class__.check_for_class_attrs(self.__class__.ATTRS_TO_CHECK)
        else:
            self.logger.warning(f"IS DEV MODE - NOT checking for class attributes "
                                f"({', '.join(self.__class__.ATTRS_TO_CHECK)}) for ContinuousMonitorAlertSend")

    def _set_args_for_endless_watch(self):
        self.send_emails = True
        self.auto_send = True
        self.display_window = False
        self.logger.debug("Configured endless_watch: send_emails=True, auto_send=True, display_window=False")

    def SetupEmail(self, recipient: Optional[str] = None, subject: str = DEFAULT_SUBJECT,
                   text: str = None, attachments: list = None, **kwargs):
        """
        :param recipient: Email recipient(s). If not provided, defaults to ADMIN_EMAIL or a semicolon-separated string of recipients in case of a list.
        :type recipient: Optional[str]
        :param subject: Subject of the email. Defaults to DEFAULT_SUBJECT.
        :type subject: str
        :param text: Body text of the email. If not provided, defaults to the response_body attribute.
        :type text: str
        :param attachments: A list of attachments to include in the email.
        :type attachments: list
        :param kwargs: Additional keyword arguments passed to the parent SetupEmail method.
        :type kwargs: dict
        :return: The resulting email setup performed by the superclass's SetupEmail method.
        :rtype: Any
        """
        if not recipient:
            recipient = self.__class__.ADMIN_EMAIL
            if isinstance(recipient, list):
                recipient = ' ;'.join(recipient)
        if not text:
            text = self.response_body
        return super().SetupEmail(recipient=recipient, subject=subject,
                                  text=text, attachments=attachments, **kwargs)

    def get_response_body_alert_level(self, msg: '_AlertMsgBase'):
        """
        :param msg: The message object which contains the alert level information.
        :type msg: _AlertMsgBase
        :return: The alert level string, optionally colorized if coloring is enabled.
        :rtype: str
        """
        if NO_COLORIZER:
            self.logger.debug("colorizer not available, using plain text for alert level")
            rb_alert_string = msg.__class__.ALERT_LEVEL.name
        else:
            self.logger.debug("colorizer available, using colorized alert level")
            color = self.colorizer.get_alert_color(msg.__class__.ALERT_LEVEL)
            rb_alert_string = self.colorizer.colorize(msg.__class__.ALERT_LEVEL.name,
                                                      color=color,
                                                      html_mode=True)
        return rb_alert_string

    @property
    def email_signature(self):
        return ('<br>'.join(super().email_signature.split('\n'))
                if super().email_signature is not None else None)

    @property
    def response_body(self):
        """
        Processes and formats the response body by compiling alert messages and their corresponding alert levels,
            then generating a formatted string containing a summary of these messages.

        :return: Processed and formatted response body string
        :rtype: str
        """
        alert_msgs = [(x.subject, self.get_response_body_alert_level(x)) for x in self.GetMessages()]
        msg_tuple = ', '.join([' - '.join(x) for x in alert_msgs])
        formatted_admin_email_names = ', '.join([x.split('@')[0] for
                                                 x in self.__class__.ADMIN_EMAIL]
                                                ).replace('\n', '<br>')
        formatted_full_body = self.__class__.DEFAULT_MSG_BODY.format(email_sender=self.email_signature,
                                                                     msg_tuple=msg_tuple,
                                                                     admin_email_names=formatted_admin_email_names
                                                                     ).replace('\n', '<br>')
        return formatted_full_body

    def _set_email_importance(self, importance_level=None, **kwargs):
        default_importance = kwargs.get('default_importance', self.__class__.DEFAULT_EMAIL_IMPORTANCE)
        try:
            if importance_level is None:
                self.email.importance = self.__class__.ALERT_EMAIL_IMPORTANCE
            else:
                self.email.importance = importance_level
        except (com_error, TypeError) as e:
            self.logger.warning(f"Invalid Importance level ({importance_level}) for email,"
                                f" setting to {default_importance}")
            self.email.importance = default_importance
            return self.email
        return self.email

    def _postprocess_alert(self, alert_level=None, **kwargs):
        self._set_email_importance(**kwargs)
        self.SendOrDisplay(**kwargs)

    def refresh_messages(self):
        self.email = self.initialize_new_email()
        self.SetupEmail()
        super().refresh_messages()


if __name__ == '__main__':
    ContinuousMonitorAlertSend.MSG_FACTORY_CLASS.ALERT_SUBJECT_KEYWORDS = ['training']
    ContinuousMonitorAlertSend.ADMIN_EMAIL = ['amcsparron@albanyny.gov']
    ContinuousMonitorAlertSend.ADMIN_EMAIL_LOGGER = ContinuousMonitorAlertSend.ADMIN_EMAIL
    cm = ContinuousMonitorAlertSend(False, False,
                                    dev_mode=False,
                                    show_warning_logs_in_console=True)  #, email_sig_filename='Andrew Full.txt')
    cm.endless_watch()



from PyEmailerAJM.continuous_monitor.backend.email_state import EmailState
from PyEmailerAJM.continuous_monitor.backend.continuous_colorizer import ContinuousColorizer
from PyEmailerAJM.continuous_monitor.backend.snooze_tracking import SnoozeTracking
from PyEmailerAJM.continuous_monitor.backend.continuous_monitor_base import ContinuousMonitorBase

__all__ = ['EmailState','ContinuousColorizer', 'SnoozeTracking', 'ContinuousMonitorBase']


from typing import Union, Tuple

from ColorizerAJM import Colorizer

from PyEmailerAJM.backend import AlertTypes


class ContinuousColorizer(Colorizer):
    """
    Class providing functionality to handle and apply color coding for alert messages.
    It extends functionality from the parent `Colorizer` class,
    adding customized color management, HTML-based color translation,
    and support for critical warning, overdue, and other alerts.

    Attributes:
      LIGHT_GRAY: ANSI escape code for light gray colored text.
      DARK_GRAY: ANSI escape code for dark gray colored text.
      ORANGE: ANSI escape code for orange colored text.
      DARK_YELLOW: ANSI escape code for dark yellow colored text.

    Methods:
      __init__(**kwargs):
        Initialize the OverdueColorizer with optional custom colors and specific default color assignments.

      get_alert_color(alert: 'AlertTypes') -> str:
        Return the appropriate color for a given alert type.

      _populate_html(unpopulated_html: str, text_for_population: str) -> str:
        Populate a provided HTML template with the given text before the closing '</span>' tag.

      colorize(text: str, color: str or None = None, bold: bool = False, **kwargs) -> str:
        Apply colorization to the provided text using ANSI escape codes or HTML formatting.

      get_color_code(color: Union[str, dict, int, Tuple[int, int, int]], **kwargs) -> str:
        Retrieve the processed color code either in plain format or as HTML, based on the settings.

      translate_color_to_html(color_code: str) -> str:
        Transform a color code into an HTML-compliant string using a `<span>` element.
    """
    ORANGE = '\x1b[33m'
    DARK_YELLOW = '\x1b[220m'

    def __init__(self, **kwargs):
        custom_colors = kwargs.pop('custom_colors', {})
        custom_colors.update({'ORANGE': self.__class__.ORANGE,
                              # DARKGOLDENROD is the color name HTML will recognize
                              'DARKGOLDENROD': self.__class__.DARK_YELLOW})
        super().__init__(custom_colors=custom_colors, **kwargs)

        self.overdue_color = self.get_color_code(self.__class__.RED, html_mode=True)
        self.critical_warning_color = self.get_color_code(self.__class__.ORANGE, html_mode=True)
        self.warning_color = self.get_color_code(self.__class__.DARK_YELLOW, html_mode=True)
        self.other_color = self.get_color_code(self.__class__.WHITE, html_mode=True)

    def get_alert_color(self, alert: 'AlertTypes'):
        """
        :param alert: The type of alert to determine the corresponding color.
        :type alert: AlertTypes
        :return: The color associated with the specified alert type.
        :rtype: str
        """
        if alert == AlertTypes.WARNING:
            return self.warning_color
        elif alert == AlertTypes.CRITICAL_WARNING:
            return self.critical_warning_color
        elif alert == AlertTypes.OVERDUE:
            return self.overdue_color
        else:
            return self.other_color

    @staticmethod
    def _populate_html(unpopulated_html: str, text_for_population: str):
        """
        :param unpopulated_html: HTML string that contains unpopulated sections marked by the closing '</span>' tag.
        :type unpopulated_html: str
        :param text_for_population: Text value to populate into the unpopulated HTML section before the closing '</span>' tag.
        :type text_for_population: str
        :return: A new HTML string with the text populated into the specified sections.
        :rtype: str
        """
        return unpopulated_html.replace('</span>', f'{text_for_population}</span>')

    def colorize(self, text, color=None, bold=False, **kwargs):
        """
        :param text: The text to be colorized.
        :type text: str
        :param color: The color to apply to the text. Defaults to None.
        :type color: str or None
        :param bold: A flag to indicate whether the text should be bold. Defaults to False.
        :type bold: bool
        :param kwargs: Additional optional parameters to control various aspects of the function. The 'html_mode' key can be used to specify whether HTML formatting is applied.
        :return: The colorized text, optionally in HTML format if 'html_mode' is enabled in kwargs.
        :rtype: str
        """
        html_mode = kwargs.get('html_mode', False)
        if html_mode:
            blank_html = self.translate_color_to_html(color)
            colorized = self._populate_html(blank_html, text)
        else:
            colorized = super().colorize(text, color=color, bold=bold)
        return colorized

    def get_color_code(self, color: Union[str, dict, int, Tuple[int, int, int]], **kwargs) -> str:
        """
        :param color: The color input which can be a string, dictionary, integer, or a tuple representing RGB values.
        :type color: Union[str, dict, int, Tuple[int, int, int]]
        :param kwargs: Additional keyword arguments. Specifically, 'html_mode' can be passed to control whether the color code is translated to HTML.
        :return: The processed color code as a string. If 'html_mode' is enabled, the color code is returned in HTML format, otherwise it's returned as is.
        :rtype: str
        """
        html_mode = kwargs.get('html_mode', False)
        if not color.startswith('\x1b') or not color.startswith('\033'):
            cc = super().get_color_code(color)
        else:
            cc = color
        if html_mode:
            return self.translate_color_to_html(cc)
        return cc

    def translate_color_to_html(self, color_code: str):
        """
        Translate a specified color code to an HTML span element.

        :param color_code: The color code in text representation. If the `color_code` starts with '<span',
                           the method attempts to map it to corresponding custom or default color codes.
                           If no mapping is found, an AttributeError is raised.
        :type color_code: str
        :return: The HTML span element string with the appropriate color style applied.
        :rtype: str
        """
        if color_code.startswith('<span'):
            new_color_code = [x[0] for x in {**self.custom_colors, **self.__class__.DEFAULT_COLOR_CODES}.items()
                              if x[1] == color_code.split('color:')[1].split('">')[0]]
            if new_color_code:
                color_code = new_color_code[0]
            else:
                raise AttributeError(f'Could not translate color code: {color_code}')
        return f'<span style="color:{color_code}"></span>'



from PyEmailerAJM.continuous_monitor.continuous_monitor import ContinuousMonitor
from PyEmailerAJM.continuous_monitor.continuous_monitor_alert_send import ContinuousMonitorAlertSend

__all__ = ['ContinuousMonitor', 'ContinuousMonitorAlertSend']


from datetime import datetime
from enum import Enum
from pathlib import Path
import inspect

import win32com.client as win32
# pylint: disable=import-error
from PyEmailerAJM.msg import Msg
from PyEmailerAJM.backend import TheSandman
from PyEmailerAJM.backend import AlertTypes


class _AlertCheckMethods:
    """
    Provides utility methods for checking specific keywords in various parts of a message,
    such as the subject, body, or attachment names.

    This class contains several class methods designed to verify whether a candidate string,
    message subject, body, or attachment names include any predefined alert keywords.

    """

    DEFAULT_ALERT_CHECK_METHOD_NAMES = ['_check_subject_for_keys',
                                        '_check_body_for_keys',
                                        '_check_attachment_name_for_keys']

    @classmethod
    def _validate_alert_check_methods(cls, alert_check_methods: list = None):
        if not alert_check_methods:
            alert_check_methods = cls.DEFAULT_ALERT_CHECK_METHOD_NAMES
        if all([callable(getattr(cls, x)) for x in alert_check_methods]):
            alert_check_methods = [getattr(cls, x) for x in alert_check_methods]
        else:
            raise AttributeError('alert_check_methods must be a list of callable objects '
                                 'that accept a single Msg argument', name=None)
        return alert_check_methods

    @classmethod
    def _check_string_for_keys(cls, candidate_string: str):
        if any((x for x in getattr(cls, 'ALERT_SUBJECT_KEYWORDS')
                if x.lower() in candidate_string.lower())):
            return True
        return False

    @classmethod
    def _check_subject_for_keys(cls, msg: Msg):
        return cls._check_string_for_keys(msg.subject)

    @classmethod
    def _check_body_for_keys(cls, msg: Msg):
        return cls._check_string_for_keys(msg.body)

    @classmethod
    def _check_attachment_name_for_keys(cls, msg: Msg):
        for a in msg.attachments:
            try:
                if cls._check_string_for_keys(Path(a).resolve().stem):
                    return True
            except (ValueError, Exception):
                continue
        return False


class _AlertMsgBase(Msg, _AlertCheckMethods):
    """
    A base class for alert message handling that inherits from ``Msg``.
    This class is designed to evaluate whether a message meets specific
    alert conditions, such as being unread, not recent, and matching predefined subject keywords.

    Attributes:
        ALERT_LEVEL (AlertTypes, optional): The alert level for the message, should be of type ``AlertTypes``.
        ALERT_TIME_HOURS (int or None): Represents the time, in hours, for an alert validity period.
            Derived from ``ALERT_LEVEL.value`` if ``ALERT_LEVEL`` is set.
        _ALERT_TIME_HOURS_ERROR (str): Message displayed when ``ALERT_TIME_HOURS`` is not properly set.

    Methods:
        __init__(self, email_item, **kwargs):
            Initializes the alert message object. Determines the cap for recent days
                from the provided or default ``ALERT_TIME_HOURS``.

        __init_subclass__(cls, **kwargs):
            Ensures subclasses properly define the ``ALERT_LEVEL`` attribute as an ``AlertTypes`` value.

        msg_alert (bool):
            Checks if the message is alert-worthy, determining conditions such as
                being unread, not recent, and containing alert keywords.

        alert_time_days (float or None):
            Retrieves the allowed alert time in days, converted from hours.

        get_alert_time_days(cls) -> float or None:
            Returns the converted alert time from hours to days at the class level.
                If errors occur during conversion, it returns ``None``.

        _msg_is_recent(self, days_limit=None) -> bool:
            Determines whether a message is recent based on a specified or default days limit.

        msg_is_rfi(cls, msg) -> bool:
            Verifies if the message subject contains any predefined RFI keywords.
    """
    ALERT_LEVEL: AlertTypes = None
    ALERT_TIME_HOURS = ALERT_LEVEL.value if ALERT_LEVEL else None
    _ALERT_TIME_HOURS_ERROR = 'ALERT_TIME_HOURS must be set to an INT value when using this class!'
    ALERT_SUBJECT_KEYWORD_ERROR = ('ALERT_SUBJECT_KEYWORDS must be a non-empty list of strings! '
                                   'Override in factory class if needed.')
    ATTRS_TO_CHECK = ['ALERT_SUBJECT_KEYWORDS']

    # noinspection PyUnresolvedReferences
    def __init__(self, email_item: win32.CDispatch or 'extract_msg.Message', **kwargs):
        if isinstance(email_item, Msg):
            email_item = email_item()

        self.logger = kwargs.get('logger', None)

        super().__init__(email_item, **kwargs)

        self.recent_days_cap = kwargs.get('recent_days_cap', self.__class__.ALERT_TIME_HOURS)
        self._msg_snoozed = None
        self._msg_snoozed_time = None
        self.snooze_checker = kwargs.get('snooze_checker', None)
        self.__class__.AlertMsgBaseCheckClsAttrs()

    def __init_subclass__(cls, **kwargs):
        if cls.ALERT_LEVEL is not None and isinstance(cls.ALERT_LEVEL, Enum):
            pass
        else:
            raise AttributeError('ALERT_LEVEL must be set to an AlertTypes or other Enum value when using this class!')

    @classmethod
    def AlertMsgBaseCheckClsAttrs(cls):
        if issubclass(cls, _AlertMsgBase):
            cls.check_for_class_attrs(cls.ATTRS_TO_CHECK)

    @classmethod
    def check_for_class_attrs(cls, class_attrs_to_check):
        for c in class_attrs_to_check:
            if hasattr(cls, c) and isinstance(getattr(cls, c), list) and len(getattr(cls, c)) > 0:
                continue
            raise AttributeError(cls.ALERT_SUBJECT_KEYWORD_ERROR)

    def _still_snoozed_check(self):
        snooze_checker_entry = self.snooze_checker.read_entry(self.subject)

        if not snooze_checker_entry and self.msg_snoozed_time:
            # FIXME: is this the cause of the "\snooze_tracking.py", line 102, in write_entry
            #  TypeError: fromisoformat: argument must be str
            snooze_expired = TheSandman.is_snooze_expired(self.msg_snoozed_time)
        elif not snooze_checker_entry and not self.msg_snoozed_time:
            snooze_expired = True
        else:
            snooze_expired = TheSandman.is_snooze_expired(snooze_checker_entry)

        if not snooze_expired:
            self.msg_snoozed = True
            return True
        self.msg_snoozed = False
        return False
        # return False

    @property
    def msg_alert(self):
        """
        Check if the message is alert worthy, meaning it is unread and not recent.

        :return: True if the email item is unread and not recent, otherwise False.
        :rtype: bool
        """
        still_snoozed = self._still_snoozed_check()
        if not still_snoozed:
            if self.email_item.Unread and not self._msg_is_recent() and self.msg_is_alert(self):
                return True
        return False

    @property
    def msg_snoozed(self):
        """
        :return: The snoozed message status.
        :rtype: bool
        """
        return self._msg_snoozed

    @msg_snoozed.setter
    def msg_snoozed(self, value: bool):
        """
        Setter method for the `msg_snoozed` property. Updates the snooze status and sets the snooze timestamp if the status is enabled.

        :param value: Indicates whether the message is snoozed. If True, sets the snooze timestamp to the current datetime, otherwise resets it to None.
        :type value: bool
        """
        self._msg_snoozed = value
        self._msg_snoozed_time = datetime.now() if value else None

    @property
    def msg_snoozed_time(self):
        """
        :return: The snoozed time of the message.
        :rtype: int or None
        """
        return self._msg_snoozed_time

    @property
    def alert_time_days(self):
        """
        :return: The alert time in days.
        :rtype: int
        """
        return self.get_alert_time_days()

    @classmethod
    def get_alert_time_days(cls):
        """
        Calculates the alert time in days based on the class attribute ALERT_TIME_HOURS.
        If ALERT_TIME_HOURS is not defined or is invalid (e.g., a type error or division by zero), returns None.

        :return: The alert time in days or None if an error occurs.
        :rtype: float or None
        """
        try:
            return cls.ALERT_TIME_HOURS / 24
        except (TypeError, ZeroDivisionError) as e:
            return None

    def _msg_is_recent(self, days_limit=None):
        """
        :param days_limit: The number of days to consider when determining if a message is recent.
            If not provided, defaults to the value of `self.recent_days_cap`.
        :type days_limit: int, optional
        :return: A boolean indicating whether the message is recent based on the given or default days limit.
        :rtype: bool
        """
        days_limit = days_limit if days_limit else self.alert_time_days
        return super()._msg_is_recent(recent_days_cap=days_limit)

    @classmethod
    def msg_is_alert(cls, msg: Msg, **kwargs):
        """
        Checks whether a given message qualifies as an alert based on specific
        validation methods.

        This method iterates over a set of alert validation methods and applies
        them to the provided message. If any of the methods validate the message
        as an alert, the method returns True. Otherwise, it returns False.

        :param msg: The message object to determine if it is an alert.
        :type msg: Msg
        :param kwargs: Additional parameters passed to alert validation methods.
        :type kwargs: dict
        :return: True if the message is an alert, False otherwise.
        :rtype: bool
        """
        alert_check_methods = [x(msg) for x in cls._validate_alert_check_methods(**kwargs)]
        if any(alert_check_methods):
            return True
        return False


class _WarningMsg(_AlertMsgBase):
    """
    A warning message class that inherits from the base alert message class.

    Attributes:
        ALERT_LEVEL: Represents the alert type, in this case, set to WARNING.
        ALERT_TIME_HOURS: Duration in hours associated with the warning level, derived from ALERT_LEVEL.
    """
    ALERT_LEVEL = AlertTypes.WARNING
    ALERT_TIME_HOURS = ALERT_LEVEL.value if ALERT_LEVEL else None


class _CriticalWarningMsg(_AlertMsgBase):
    """
    Represents a critical warning alert message.

    This class is a specialization of the `_AlertMsgBase` class tailored to handle
    critical warning alerts. It defines specific attributes such as `ALERT_LEVEL`
    and `ALERT_TIME_HOURS`, which determines the severity and time-related settings
    of the alert.

    Attributes
    ----------
    ALERT_LEVEL : AlertTypes
        A constant representing the severity level of this alert, set to
        `AlertTypes.CRITICAL_WARNING`.
    ALERT_TIME_HOURS : int or None
        The time duration associated with this alert in hours. Derived from the
        value of `ALERT_LEVEL` if available, otherwise `None`.
    """
    ALERT_LEVEL = AlertTypes.CRITICAL_WARNING
    ALERT_TIME_HOURS = ALERT_LEVEL.value if ALERT_LEVEL else None


class _OverDueMsg(_AlertMsgBase):
    """
    Represents a specialized message object that determines if a message is overdue
        or related to a Request For Information (RFI).

    Attributes:
        ALERT_SUBJECT_KEYWORDS: List of keywords used to identify if a message
            subject relates to a Request For Information (RFI).

    Methods:
        __init__(email_item, **kwargs):
            Initializes the _OverDueMsg object with an email item and optional parameters such as recent_days_cap.

        _msg_is_recent(days_limit=None):
            Determines whether the message is recent based on the number of days provided
                or the default recent_days_cap.

    Properties:
        msg_alert:
            Checks if the message is unread and not recent, marking it as overdue.

        msg_is_rfi:
            Checks if the message subject matches any of the predefined RFI keywords, marking it as related to an RFI.
    """
    ALERT_LEVEL = AlertTypes.OVERDUE
    ALERT_TIME_HOURS = ALERT_LEVEL.value if ALERT_LEVEL else None



from importlib import import_module


def is_instance_of_dynamic(obj: object, base_class_path: str) -> bool:
    """
    Check if an object is an instance of a class or its subclass specified by its module path.
    """
    try:
        module_path, class_name = base_class_path.rsplit('.', 1)
        module = import_module(module_path)
        base_class = getattr(module, class_name)
        return isinstance(obj, base_class)
    except (ImportError, AttributeError):
        return False


from PyEmailerAJM.backend import deprecated
from PyEmailerAJM.backend.errs import EmailerNotSetupError, DisplayManualQuit
from PyEmailerAJM.msg import Msg, FailedMsg
from PyEmailerAJM.searchers import SearcherFactory
from PyEmailerAJM.py_emailer_ajm import PyEmailer, EmailerInitializer
from PyEmailerAJM.continuous_monitor.continuous_monitor import ContinuousMonitor

__all__ = ['EmailerNotSetupError', 'DisplayManualQuit', 'deprecated',
           'Msg', 'FailedMsg', 'PyEmailer', 'EmailerInitializer',
           'SearcherFactory', 'ContinuousMonitor',
           'is_instance_of_dynamic']




from abc import abstractmethod
from typing import Callable

from PyEmailerAJM.backend import AlertTypes
from PyEmailerAJM.continuous_monitor.backend.continuous_monitor_base import ContinuousMonitorBase
from PyEmailerAJM.msg import MsgFactory


class ContinuousMonitor(ContinuousMonitorBase):
    TITLE_STRING = " Watching for emails with alerts in {} folder ".center(100, '*')
    MSG_FACTORY_CLASS: MsgFactory = MsgFactory
    ALERT_CHECK_STR = "Checking for emails with an alert..."
    NO_ALERTS_STR = "No emails with an alert detected in {read_folder} ({num_snoozed} snoozed)."

    def GetMessages(self, folder_index=None):
        """
        :param folder_index: Index of the folder from which messages are retrieved. Defaults to None if not specified.
        :type folder_index: int, optional
        :return: A list of sorted and filtered message objects, each containing an alert.
        :rtype: list
        """
        msgs = super().GetMessages(folder_index)
        sorted_msgs = [
            self.__class__.MSG_FACTORY_CLASS.get_msg(x, logger=self.logger, snooze_checker=self.snooze_tracker) for x in
            msgs]
        alert_messages = [x for x in sorted_msgs if x is not None and x.msg_alert]
        return alert_messages

    def _set_args_for_endless_watch(self):
        """
        Sets specific arguments for the endless_watch process.

        :return: None
        :rtype: None
        """
        self.send_emails = False
        self.auto_send = False
        self.display_window = False

    @abstractmethod
    def _postprocess_alert(self, alert_level=None, **kwargs):
        ...

    def _process_no_alert(self, **kwargs):
        no_alert_str = kwargs.get('no_alert_string',
                                  self.__class__.NO_ALERTS_STR.format(read_folder=self.read_folder,
                                                                      num_snoozed=self.num_snoozed_msgs))
        self.logger.info(no_alert_str, print_msg=True)

    def _classify_and_process(self, **kwargs):
        if self.has_overdue:
            self._print_and_postprocess(AlertTypes.OVERDUE)

        elif self.has_warning:
            self._print_and_postprocess(AlertTypes.WARNING)

        elif self.has_critical_warning:
            self._print_and_postprocess(AlertTypes.CRITICAL_WARNING)

        else:
            self._process_no_alert(**kwargs)

    # TODO: make no_alerts_string property so there is more flexibility with format
    def check_for_alerts(self, **kwargs):
        """
        Checks for emails in the specified folder and identifies if there are any alerts. Alerts,
        if present, are categorized as overdue, warning, or critical warning, and are processed accordingly.
        Then logs the result of the check.

        :return: None
        :rtype: None

        """
        alert_check_string = kwargs.get('alert_check_string', self.__class__.ALERT_CHECK_STR)
        self.logger.info(alert_check_string, print_msg=True)
        self.refresh_messages()

        self._classify_and_process(**kwargs)

        self.snooze_tracker.snooze_msgs(self.all_messages)

    def endless_watch(self, stop_condition: Callable[[], bool] = None):
        if not self.dev_mode:
            self._set_args_for_endless_watch()

        stop_condition = stop_condition or (lambda: False)  # Default stop_condition
        email_dir_name = self.read_folder.name if self.read_folder else None

        self.logger.info(self.__class__.TITLE_STRING.format(email_dir_name), print_msg=True)

        while not stop_condition():
            try:
                self.check_for_alerts()
                self._was_refreshed = False
                self.sleep_timer.sleep_in_rounds()
            except KeyboardInterrupt:
                self.logger.error("KeyboardInterrupt detected, exiting program.")
                break


if __name__ == '__main__':
    ContinuousMonitor.MSG_FACTORY_CLASS.ALERT_SUBJECT_KEYWORDS = ['training']
    cm = ContinuousMonitor(False, False, dev_mode=False, show_warning_logs_in_console=True, )
    cm.endless_watch()
